﻿-- *****************
-- * Options Panel *
-- *****************

-- declare these BEFORE anything that depends on LFMonitorDB integrity!
function LFMonitor:ResponseOptionsDefault(CreateOptionsAlreadyRunning)

	LFMonitor:ClearAlerts()
	LFMonitorDB = LFMonitor:deepcopy(LFMonitor.defaults)
	LFMonitorCache = LFMonitor:deepcopy(LFMonitor.defaultCache)
	LFMonitorWordCache = LFMonitor:deepcopy(LFMonitor.defaultCache)
	LFMonitor:SetAlertSkin()
	LFMonitor:ClearAlerts()
	LFMonitor:BuildTables()
	if not CreateOptionsAlreadyRunning then
		LFMonitor:CreateOptions()
		LFMonitor:ChatCommand(nil) -- can't run during CreateOptions because our main panel doesn't exist or needs to be rebuilt
	end

end

function LFMonitor:CheckSettingsVersion(CreateOptionsAlreadyRunning)
	if (not LFMonitorDB.settingsVersion) or LFMonitorDB.settingsVersion < LFMonitor.settingsVersion then
		LFMonitor:ResponseOptionsDefault(CreateOptionsAlreadyRunning)
		message(LFMonitor.LOCALE_STRINGS["Your settings have been reset because they were incompatible with this version of LFMonitor."])
	elseif (not LFMonitorCache.version) or LFMonitorCache.version < LFMonitor.cacheVersion then
		LFMonitorCache.count = 0
		LFMonitorCache.score = {}
		LFMonitorCache.version = LFMonitor.cacheVersion
	end
end

local _G = _G
local bz = LFMonitor.BZ
local bi = LFMonitor.BI
local l = string.lower
local loc = LFMonitor.LOCALE_STRINGS
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

LFMonitor.GeneralOptionTable = {
	name		= _G.GENERAL,
	type		= "group",
	args		= {
		channels = {
			name	= loc["Monitored channels"],
			order   = 0,
			type	= "multiselect",
			get		= function(_,key) return LFMonitorDB.channels[key] or false end,
			set		= function(_,key, val) LFMonitorDB.channels[key] = val end,
			values	= {
				general	= _G.GENERAL,
				trade	= _G.TRADE,
				guild	= _G.GUILD,
				lfg		= _G.LFG_TITLE
			},
		},
		whentoshow = {
			name   = loc["Hide LFMonitor when:"],
			order  = 1,
			type   = "multiselect",
			get	   = function(_,key) return LFMonitorDB.options[key] or false end,
			set	   = function(_,key, val)
				LFMonitorDB.options[key] = val
				LFMonitor:DisableCheck()
			end,
			values = {
				disabled = _G.ALWAYS,
				hinst = _G.AGGRO_WARNING_IN_INSTANCE,
				trade = _G.CHAT_NOT_MEMBER_NOTICE:format(_G.TRADE), -- "Not on channel Trade."
				rested = loc["Not in a city or inn"],
				combat = loc["In combat"],
			},
		},
		lock = {
			name	= _G.LOCK_FRAME,
			order	= 2,
			type	= "toggle",
			get		= function() return LFMonitorDB.options.locked end,
			set		= function(_,val) LFMonitorDB.options.locked = val end,
		},
		ignoreselfcheck = {
			name	= loc["Ignore my own messages"],
			order   = 3,
			width	= "double",
			type	= "toggle",
			get		= function() return LFMonitorDB.options.ignoreself or false end,
			set		= function(_,val) LFMonitorDB.options.ignoreself = val end,
		},
		cache = {
			name	= loc["Cache size"],
			type	= "range",
			min		= 10,
			max		= 20,
			get		= function() return math.log(LFMonitorCache.max)/math.log(2) end,
			set		= function(_,val)
				LFMonitorCache = LFMonitor:deepcopy(LFMonitor.defaultCache)
				LFMonitorCache.max = 2^val
				LFMonitorWordCache = LFMonitor:deepcopy(LFMonitor.defaultCache)
				LFMonitorCache.max = 2^val
			end,
		},
		sensitivity = {
			name	= loc["Filter sensitivity"],
			type	= "range",
			min		= 0,
			max		= 1,
			softMax = 0.9,
			bigStep = 0.1,
			isPercent = true,
			get		= function() return LFMonitorDB.options.sensitivity end,
			set		= function(_,val) LFMonitorDB.options.sensitivity = val end,
		},
		sound = {
			name	= _G.SOUND_LABEL,
			type	= "select",
			values	= {
				loc["Coins"],
				loc["Window tab"],
				loc["Menu opening"],
				loc["Minimap opening"],
				loc["Spellbook opening"],
				loc["Spellbook closing"],
				loc["Talents opening"],
				loc["Quest log opening"],
				bi["Fishing"],
				loc["Discovery"],
				loc["Applause"],
				loc["Beep"],
				loc["Click"],
				loc["Dog barking"],
				loc["Mouse click"],
				loc["Stapler"],
				[0] = _G.NONE,
			},
			get	= function()
				if LFMonitorDB.options.soundon then
					return LFMonitorDB.options.sound
				else
					return 0
				end
			end,
			set = function(_,val)
				if val == 0 then
					LFMonitorDB.options.soundon = false
				else
					LFMonitorDB.options.soundon = true
					LFMonitorDB.options.sound = val
				end
			end,
		},
	},
}


local profslist = {
	["jc"] = bi["Jewelcrafting"],
	["enc"] = bi["Enchanting"],
	["lw"] = bi["Leatherworking"],
	["ins"] = bi["Inscription"],
	["tlr"] = bi["Tailoring"],
	["bs"] = bi["Blacksmithing"],
	["alc"] = bi["Alchemy"],
	["pm"] = GetSpellInfo(28675), --Potion Master
	["em"] = GetSpellInfo(28677), --Elixir Master
	["xm"] = GetSpellInfo(28672), --Transmutation Master
	["eng"] = bi["Engineering"],
	["sm"] = GetSpellInfo(2656), --Smelting
}

local serviceslist = {
	["co"] = bi["Cooking"],
	["lp"] = GetSpellInfo(6460), --Lockpicking
	["port"] = loc["Mage Portals"],
	["hgwt"] = GetSpellLink(83967), -- Have Group, Will Travel
	["fly"] = loc["Mount Rides"],
	["misc"] = _G.MISCELLANEOUS,
}

local rolelist = {
	["tank"] = loc["Tank / Flag Carrier"],
	["healer"] = _G.HEALER,
	["melee"] = _G.PLAYERSTAT_MELEE_COMBAT,
	["ranged"] = _G.PLAYERSTAT_RANGED_COMBAT,
	["cc"] = loc["Crowd Control"]
}

local arenalist = {
	["2s"]  = _G.ARENA_2V2,
	["3s"]  = _G.ARENA_3V3,
	["5s"]  = _G.ARENA_5V5,
}
local bglist = {
	["RBG"] = _G.PVP_RATED_BATTLEGROUNDS,
}
local dunglist = {
	["ZA"] = bz["Zul'Aman"],
	["ZG"] = bz["Zul'Gurub"]
}
local catalist = {
	["FL"]	= bz["Firelands"],
	["BH"]	= bz["Baradin Hold"],
	["BWD"] = bz["Blackwing Descent"],
	["BoT"]	= bz["The Bastion of Twilight"],
	["TFW"]	= bz["Throne of the Four Winds"],
}
local wrathlist = {
	["EoE"] = bz["The Eye of Eternity"],
	["ICC"] = bz["Icecrown Citadel"],
	["Nax"] = bz["Naxxramas"],
	["Ony"] = bz["Onyxia's Lair"],
	["OS"] = bz["The Obsidian Sanctum"],
	["RS"] = bz["The Ruby Sanctum"],
	["ToC"] = bz["Trial of the Crusader"],
	["Uld"] = bz["Ulduar"],
	["VoA"] = bz["Vault of Archavon"],
}
local bclist = {
	["Hyj"] = bz["Mount Hyjal"],
	["BT"] = bz["Black Temple"],
	["GL"] = bz["Gruul's Lair"],
	["Kra"] = bz["Karazhan"],
	["Mag"] = bz["Magtheridon's Lair"],
	["SSC"] = bz["Serpentshrine Cavern"],
	["SWP"] = bz["Sunwell Plateau"],
	["TK"] = bz["Tempest Keep"],
}
local classiclist = {
	["BWL"] = bz["Blackwing Lair"],
	["MC"] = bz["Molten Core"],
	["AQ10"] = bz["Ruins of Ahn'Qiraj"],
	["AQ40"] = bz["Temple of Ahn'Qiraj"],
	["UBRS"] = bz["Upper Blackrock Spire"],
}

local function raidsArgs(type)
	local function getter(_,key)
		return LFMonitorDB[type].raids[key] or false
	end
	local function setter(_,key,val)
		LFMonitorDB[type].raids[key] = val
	end

	return {instances={
		name = loc["Instances"],
		type = "group",
		order = -1,
		args = {
			override={
				name	= loc["Don't filter by instance"],
				type	= "toggle",
				get		= function() return LFMonitorDB[type].overrideraid end,
				set		= function(_,val) LFMonitorDB[type].overrideraid = val end,
			},
			pvp = {
				width	= "double",
				name	= _G.PLAYER_V_PLAYER,
				type	= "group",
				inline	= false,
				order	= 0,
				args	= {
					arenas = {
						name	= _G.ARENA_TEAMS,
						type	= "multiselect",
						values	= arenalist,
						get		= getter,
						set		= setter,
					},
					bgs = {
						name	= _G.BATTLEFIELDS,
						type	= "multiselect",
						values	= bglist,
						get		= getter,
						set		= setter,
					}
				}
			},
			raids = {
				width	= "double",
				name	= loc["Raids"],
				type	= "group",
				inline	= false,
				order	= 1,
				args	= {
					cata = {
						width	= "double",
						name	= _G.EXPANSION_NAME3,
						type	= "multiselect",
						values	= catalist,
						get		= getter,
						set		= setter,
						order	= 97,
					},
					wrath = {
						width	= "double",
						name	= _G.EXPANSION_NAME2,
						type	= "multiselect",
						values	= wrathlist,
						get		= getter,
						set		= setter,
						order	= 98,
					},
					bc = {
						width	= "double",
						name	= _G.EXPANSION_NAME1,
						type	= "multiselect",
						values	= bclist,
						get		= getter,
						set		= setter,
						order	= 99,
					},
					classic = {
						width	= "double",
						name	= _G.EXPANSION_NAME0,
						type	= "multiselect",
						values	= classiclist,
						get		= getter,
						set		= setter,
						order	= 100,
					},
				},
			},
			dungeons = {
				width	= "double",
				name	= loc["Dungeons"],
				type	= "group",
				inline	= false,
				order	= 2,
				args	= {
					cata = {
						name	= _G.EXPANSION_NAME3,
						width	= "double",
						type	= "multiselect",
						values	= dunglist,
						get		= getter,
						set		= setter,
						order	= 2,
					}
				}
			},
		},
	}}
end

local function profsArgs(type)
	local function getter(_,key)
		return LFMonitorDB[type].professions[key] or false
	end
	local function setter(_,key,val)
		LFMonitorDB[type].professions[key] = val
	end
	return {
			profs={
				name	= _G.TRADE_SKILLS,
				type 	= "multiselect",
				values	= profslist,
				get		= getter,
				set		= setter,
				order	= 10,
			},
			services={
				name	= loc["Other Services"],
				type 	= "multiselect",
				values	= serviceslist,
				get		= getter,
				set		= setter,
				order	= 11,
			}
	}
end

local function classRoleArgs(type)
	return {classroles={
		name	= loc["Classes"] .. "/" .. loc["Roles"],
		type	= "group",
		order	= 20,
		args	= {
			roles={
				name 	= loc["Roles"],
				type 	= "multiselect",
				values 	= rolelist,
				get		= function(_,key) return LFMonitorDB[type].roles[key] or false end,
				set		= function(_,key, val) LFMonitorDB[type].roles[key] = val end,
				order	= 30,
			},
			classes={
				name 	= loc["Classes"],
				type 	= "multiselect",
				values 	= _G.LOCALIZED_CLASS_NAMES_MALE,
				get		= function(_,key) return LFMonitorDB[type].classes[key] or false end,
				set		= function(_,key, val) LFMonitorDB[type].classes[key] = val end,
				order	= 20,
			},
		}
	}}
end

local function createTab(type, longname, extrafuncs, inline)
	local tab = {
		type	= "group",
		inline	= inline or false,
		childGroups = "tree",
		args    = {
			header = {
				name   = longname,
				type   = "header",
				order  = 0,
				hidden = inline or false
			},
			enable = {
				name	= _G.ENABLE,
				desc	= loc["Search for and display messages of this type"],
				type	= "toggle",
				order   = 1,
				get		= function () return LFMonitorDB.monitoring[type] or false end,
				set		= function (_,val) LFMonitorDB.monitoring[type] = val end,
			}
		},
		plugins = {}
	}
	if inline then
		tab.name	= longname
	else
		tab.name	= LFMonitor.ABBRS.type[type]
		tab.desc	= longname
	end
	local k,v
	for k,v in ipairs(extrafuncs) do
		tab.plugins[""..k] = v(type)
	end
	return tab
end

LFMonitor.FilterOptionTable = {
  name			= loc["Filtering"],
  type			= "group",
  childGroups	= "tab",
  args 			= {
	lfw		= createTab("lfw",		loc["Looking for Work"],			{profsArgs}),
	lfprof	= createTab("lfprof",	loc["Looking for Crafter/Service"], {profsArgs}),
	lfg		= createTab("lfg",		_G.LFG_TITLE,						{classRoleArgs,raidsArgs}),
	lfm		= createTab("lfm",		_G.LFM_TITLE,						{classRoleArgs,raidsArgs}),
	lfguild	= createTab("lfguild",	loc["Looking for Guild"],			{classRoleArgs}),
	guildlfm= createTab("guildlfm",	loc["Guild Recruitment"],			{classRoleArgs}),
	wtbwts  = {
		name	= LFMonitor.ABBRS.type["wtb"] .. " / " .. LFMonitor.ABBRS.type["wts"],
		desc	= loc["Want to Buy"] .. " / " .. loc["Want to Sell"],
		type	= "group",
		args	= {
			wtb = createTab("wtb", loc["Want to Buy"], {}, true),
			wts = createTab("wts", loc["Want to Sell"], {}, true),
		}
	}
  }
}


local function OptionColour(frame, entity)

	local r, g, b = ColorPickerFrame:GetColorRGB()
	local ir, ig, ib, ia

	if ((entity == "timestamp") or (entity == "player") or (entity == "text")) then

		ir, ig, ib, ia = LFMonitor:HexToRGB(LFMonitorDB.alert.font[entity])
		LFMonitorDB.alert.font[entity] = LFMonitor:RGBToHex(r, g, b, ia)

	else

		ir, ig, ib, ia = LFMonitor:HexToRGB(LFMonitorDB.alert.backdrop[entity])
		LFMonitorDB.alert.backdrop[entity] = LFMonitor:RGBToHex(r, g, b, ia)
		LFMonitor:SetAlertSkin()

	end

	frame:SetTexture(r, g, b, 1)

end

local function OptionOpacity(frame, entity)

	local a = 1 - OpacitySliderFrame:GetValue()
	local ir, ig, ib, ia

	if ((entity == "timestamp") or (entity == "player") or (entity == "text")) then

		ir, ig, ib, ia = LFMonitor:HexToRGB(LFMonitorDB.alert.font[entity])
		LFMonitorDB.alert.font[entity] = LFMonitor:RGBToHex(ir, ig, ib, a)

	else

		ir, ig, ib, ia = LFMonitor:HexToRGB(LFMonitorDB.alert.backdrop[entity])
		LFMonitorDB.alert.backdrop[entity] = LFMonitor:RGBToHex(ir, ig, ib, a)
		LFMonitor:SetAlertSkin()
	end

end

local function OptionCancel(frame, entity)

	local r, g, b, a = unpack(ColorPickerFrame.previousValues)

	if ((entity == "timestamp") or (entity == "player") or (entity == "text")) then

		LFMonitorDB.alert.font[entity] = LFMonitor:RGBToHex(r, g, b, a)

	else

		LFMonitorDB.alert.backdrop[entity] = LFMonitor:RGBToHex(r, g, b, a)
		LFMonitor:SetAlertSkin()

	end

	frame:SetTexture(r, g, b, 1)

end

function LFMonitor:ResponseOptionsOkay()

	if(_G.LFMonitorOptionsRespond:IsVisible())then

		LFMonitorDB.responses.alc = _G.LFMonitorAlcEdit:GetText()
		LFMonitorDB.responses.bs = _G.LFMonitorBsEdit:GetText()
		LFMonitorDB.responses.enc = _G.LFMonitorEncEdit:GetText()
		LFMonitorDB.responses.eng = _G.LFMonitorEngEdit:GetText()
		LFMonitorDB.responses.ins = _G.LFMonitorInsEdit:GetText()
		LFMonitorDB.responses.jc = _G.LFMonitorJcEdit:GetText()
		LFMonitorDB.responses.lw = _G.LFMonitorLwEdit:GetText()
		LFMonitorDB.responses.tlr = _G.LFMonitorTlrEdit:GetText()
		LFMonitorDB.responses.lfm = _G.LFMonitorLFGEdit:GetText()

	end

	LFMonitor:BuildTables()

end

function LFMonitor:CustomOptionsOkay()

	local bas = _G.LFMonitorBasicEdit:GetText() or ""
	local adv = _G.LFMonitorAdvEdit:GetText() or ""

	if(_G.LFMonitorOptionsCustom:IsVisible())then

		LFMonitorDB.options.basic = bas

		if (bas ~= "") then

			-- make sure magic characters are escaped to preserve literal values
			-- ^$().[]*+-?%
			bas = string.gsub(bas, "[%^%$%(%)%.%[%]%*%+%-%?%%]", "%%%1")

			--strip multiple spaces
			bas = string.gsub(bas, "%s+", " ")

		end

		LFMonitorDB.options.basicesc = bas
		LFMonitorDB.options.adv = adv

	end

end

local function SetSelectColour(frame, entity)

	local r, g, b, a

	if ((entity == "background") or (entity == "border")) then

		r, g, b, a = LFMonitor:HexToRGB(LFMonitorDB.alert.backdrop[entity])

	else

		r, g, b, a =  LFMonitor:HexToRGB(LFMonitorDB.alert.font[entity])

	end

	frame:SetTexture(r, g, b, 1)

end

function LFMonitor:FontSliderOnValueChanged(self)

	local slider = self:GetName()

	LFMonitorDB.alert.font.height = self:GetValue()
	LFMonitor:SetAlertSkin()
	_G[slider .. "Text"]:SetText(LFMonitorDB.alert.font.height)

end


local function CreateMainPanel()

	local frame = CreateFrame("Frame", "LFMonitorOptionsMain")
	local title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	local version = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	local author = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")

	title:SetText("LFMonitor")
	title:SetPoint("CENTER", frame, "CENTER", 0, 170)

	version:SetText(_G.GAME_VERSION_LABEL .. " " .. GetAddOnMetadata("LFMonitor", "Version"))
	version:SetPoint("CENTER", frame, "CENTER", 0, 140)

	author:SetText("Authors:\nDeepac\nSeahen (Arcanedamage - Blackrock (US))")
	author:SetPoint("CENTER", frame, "CENTER", 0, 110)

	return frame

end

local exampletime = time{year=2004,month=11,day=23,hour=13,min=37,sec=00}
LFMonitor.AppearanceOptionTable = {
	name = _G.APPEARANCE_LABEL,
	type = "group",
	args = {
		colors = {
			name   = _G.COLORS,
			type   = "group",
			inline = true,
			args   = {
				font_text = {
					name 		= loc["Message Text"],
					type		= "color",
				},
				font_timestamp = {
					name 		= _G.TIMESTAMPS_LABEL,
					type 		= "color",
				},
				font_player = {
					name		= _G.CALENDAR_PLAYER_NAME,
					type 		= "color",
				},
				backdrop_background = {
					name 		= _G.EMBLEM_BACKGROUND,
					hasAlpha	= true,
					type		= "color",
				},
				backdrop_border = {
					name		= _G.EMBLEM_BORDER,
					hasAlpha	= true,
					type		= "color",
				},
				classcolors = {
					name = _G.RAID_USE_CLASS_COLORS,
					type = "toggle",
					get  = function()
						return LFMonitorDB.options.useclasscolours or false
					end,
					set  = function(_,val)
						LFMonitorDB.options.useclasscolours = val
						LFMonitor:SetAlertSkin()
					end,
				},
			},
			get = function(item)
				local entity, subentity = string.split("_",item[#item])
				local r, g, b, a = LFMonitor:HexToRGB(LFMonitorDB.alert[entity][subentity])
				return r,g,b,a
			end,
			set = function(item,r,g,b,a)
				local entity, subentity = string.split("_",item[#item])
				LFMonitorDB.alert[entity][subentity] = LFMonitor:RGBToHex(r,g,b,a)
				LFMonitor:SetAlertSkin()
			end,
		},
		font = {
			name = loc["Font"],
			type = "select",
			values	= {
				loc["Friz Quadrata TT"],
				loc["Arial Narrow"],
				loc["Skurri"],
				loc["Morpheus"],
			},
			get = function()
				return LFMonitorDB.alert.font.index
			end,
			set = function(_,val)
				LFMonitorDB.alert.font.index = val
				LFMonitor:SetAlertSkin()
			end,
		},
		fontsize = {
			name = _G.FONT_SIZE,
			type = "range",
			min  = 6,
			max  = 18,
			step = 1,
			get = function()
				return LFMonitorDB.alert.font.height
			end,
			set = function(_,val)
				LFMonitorDB.alert.font.height = val
				LFMonitor:SetAlertSkin()
			end,
		},
		timestamps = {
			name = _G.TIMESTAMPS_LABEL,
			type = "select",
			values = {
				[_G.TIMESTAMP_FORMAT_HHMM.." "]			= date(_G.TIMESTAMP_FORMAT_HHMM,exampletime),
				[_G.TIMESTAMP_FORMAT_HHMMSS.." "]		= date(_G.TIMESTAMP_FORMAT_HHMMSS,exampletime),
				[_G.TIMESTAMP_FORMAT_HHMM_AMPM.." "]	= date(_G.TIMESTAMP_FORMAT_HHMM_AMPM,exampletime),
				[_G.TIMESTAMP_FORMAT_HHMMSS_AMPM.." "]	= date(_G.TIMESTAMP_FORMAT_HHMMSS_AMPM,exampletime),
				[_G.TIMESTAMP_FORMAT_HHMM_24HR.." "]	= date(_G.TIMESTAMP_FORMAT_HHMM_24HR,exampletime),
				[_G.TIMESTAMP_FORMAT_HHMMSS_24HR.." "]	= date(_G.TIMESTAMP_FORMAT_HHMMSS_24HR,exampletime),
				[""]									= _G.TIMESTAMP_FORMAT_NONE,
			},
			get	= function()
				return LFMonitorDB.options.timestamp or ""
			end,
			set	= function(_,val)
				LFMonitorDB.options.timestamp = val
			end,
		},
	},
}

local function CreateResponsePanel()

	local frame = CreateFrame("Frame", "LFMonitorOptionsRespond")
	local alcedit = CreateFrame("EditBox", "LFMonitorAlcEdit", frame, "InputBoxTemplate")
	local bsedit = CreateFrame("EditBox", "LFMonitorBsEdit", frame, "InputBoxTemplate")
	local encedit = CreateFrame("EditBox", "LFMonitorEncEdit", frame, "InputBoxTemplate")
	local engedit = CreateFrame("EditBox", "LFMonitorEngEdit", frame, "InputBoxTemplate")
	local insedit = CreateFrame("EditBox", "LFMonitorInsEdit", frame, "InputBoxTemplate")
	local jcedit = CreateFrame("EditBox", "LFMonitorJcEdit", frame, "InputBoxTemplate")
	local lwedit = CreateFrame("EditBox", "LFMonitorLwEdit", frame, "InputBoxTemplate")
	local tlredit = CreateFrame("EditBox", "LFMonitorTlrEdit", frame, "InputBoxTemplate")
	local lfgedit = CreateFrame("EditBox", "LFMonitorLFGEdit", frame, "InputBoxTemplate")
	local title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	local alctext = alcedit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local bstext = bsedit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local enctext = encedit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local engtext = engedit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local instext = insedit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local jctext = jcedit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local lwtext = lwedit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local tlrtext = tlredit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local lfgtext = lfgedit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local edits = {
		{box = alcedit, text = alctext, db = "alc", title = LFMonitor.ALCHEMY},
		{box = bsedit, text = bstext, db = "bs", title = LFMonitor.BLACKSMITHING},
		{box = encedit, text = enctext, db = "enc", title = LFMonitor.ENCHANTING},
		{box = engedit, text = engtext, db = "eng", title = LFMonitor.ENGINEERING},
		{box = insedit, text = instext, db = "ins", title = LFMonitor.INSCRIPTION},
		{box = jcedit, text = jctext, db = "jc", title = LFMonitor.JEWELCRAFTING},
		{box = lwedit, text = lwtext, db = "lw", title = LFMonitor.LEATHERWORKING},
		{box = tlredit, text = tlrtext, db = "tlr", title = LFMonitor.TAILORING},
		}
	local first, anchor = true, title
	local offsety, offsetx = -55, 10

	frame:Hide()

	title:SetPoint("TOPLEFT", frame, "TOPLEFT", 5, -20)
	title:SetWidth(400)
	title:SetText(LFMonitor.RESPONSES_TO_REQUESTS)

	local function responseOnShow()

		for _, edit in ipairs(edits) do

			if first then

				anchor = title
				first = nil
				offsetx = 10

			else

				offsetx = 0

			end

				edit.box:SetPoint("LEFT", anchor, "TOPLEFT", offsetx, offsety)
				edit.box:SetWidth(385)
				edit.box:SetHeight(32)
				edit.box:SetMaxLetters(90)
				edit.box:SetAutoFocus(false)
				edit.box:SetText(LFMonitorDB.responses[edit.db])
				edit.box:SetScript("OnEditFocusGained", function(self) LFMonitor.EditFocus = self end)
				edit.box:SetScript("OnEditFocusLost", function() LFMonitor.EditFocus = nil end)
				edit.text:SetPoint("LEFT", edit.box, "TOPLEFT", 0, 5)
				edit.text:SetText(edit.title)
				edit.box:Show()
				anchor = edit.box

		end

		if (LFMonitorDB.monitoring.lfm == 1) then

			local function lfgMouse(self, button)

				local cursor = self:GetCursorPosition()
				local text = self:GetText()
				local segment = string.sub(text, 1, cursor)
				local link, txt, sp
				local ls, le = string.find(segment, "|c.-|h|r")

				if ((cursor == nil) or (cursor >= string.len(text))) then

					return

				end

				--remove any leading links and find the one we want

				sp = ls or 1

				while (le) do

					segment = string.sub(segment, le +1)
					sp = sp + le
					ls, le = string.find(segment, "|c.-|h|r")

				end

				ls, le = string.find(segment, "|c")

				if (ls) then

					txt = string.match(text, "|c.-|h|r", sp)
					link = string.match(text, "|H(.-)|h", sp)
					SetItemRef(link, txt, button)

				end

			end

			lfgedit:SetPoint("LEFT", anchor, "TOPLEFT", offsetx, offsety)
			lfgedit:SetWidth(385)
			lfgedit:SetHeight(32)
			lfgedit:SetMaxLetters(90)
			lfgedit:SetAutoFocus(false)
			lfgedit:SetScript("OnEditFocusGained", function(self) LFMonitor.EditFocus = self end)
			lfgedit:SetScript("OnEditFocusLost", function() LFMonitor.EditFocus = nil end)
			lfgedit:SetScript("OnMouseUp", lfgMouse)

			if (LFMonitorDB.responses.lfm == nil) then

				LFMonitorDB.responses.lfm = LFMonitor.LFG_RESPONSE

			end

			lfgedit:SetText(LFMonitorDB.responses.lfm)
			lfgtext:SetPoint("LEFT", lfgedit, "TOPLEFT", 0, 5)
			lfgtext:SetText(_G.LFG_TITLE)
			lfgedit:Show()

		else

			lfgedit:Hide()

		end

		first = true

	end

	local function responseOnHide()

		for _, edit in ipairs(edits) do

			edit.box:ClearAllPoints()
			edit.text:ClearAllPoints()

		end

	end

	frame:SetScript("OnShow", responseOnShow)
	frame:SetScript("OnHide", responseOnHide)

	return frame

end

local function CreateCustomPanel()

	local frame = CreateFrame("Frame", "LFMonitorOptionsCustom")
	local basicedit = CreateFrame("EditBox", "LFMonitorBasicEdit", frame, "InputBoxTemplate")
	local advedit = CreateFrame("EditBox", "LFMonitorAdvEdit", frame, "InputBoxTemplate")
	local title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	local basictext = basicedit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local advtext = advedit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local inst = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local warn = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	LFMonitor.cchantext = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	LFMonitor.cc1 = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	LFMonitor.cc1text = LFMonitor.cc1:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	LFMonitor.cc2 = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	LFMonitor.cc2text = LFMonitor.cc2:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	LFMonitor.cc3 = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	LFMonitor.cc3text = LFMonitor.cc3:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	LFMonitor.cc4 = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	LFMonitor.cc4text = LFMonitor.cc4:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	LFMonitor.cc5 = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	LFMonitor.cc5text = LFMonitor.cc5:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	LFMonitor.cc6 = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	LFMonitor.cc6text = LFMonitor.cc6:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	LFMonitor.cc7 = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	LFMonitor.cc7text = LFMonitor.cc7:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local ctl

	local function customOnShow()

		local i, v
		local channels = {GetChannelList()}

		v = 1

		for i = 1, 7 do

			LFMonitor["cc" .. i]:Hide()

		end

		for i = 1, #channels, 2 do

			if (channels[i] > 4) then

				LFMonitor["cc" .. v .. "text"]:SetText(channels[i + 1])
				LFMonitor["cc" .. v]:SetChecked(LFMonitorDB.channels["cc" .. channels[i + 1]])
				LFMonitor["cc" .. v]:Show()
				v = v + 1

			end

		end

		if (v == 1) then

			LFMonitor.cchantext:Hide()

		else

			LFMonitor.cchantext:Show()

		end

	end

	frame:Hide()
	frame:SetScript("OnShow", customOnShow)

	title:SetPoint("TOPLEFT", frame, "TOPLEFT", 5, -20)
	title:SetWidth(400)
	title:SetText(LFMonitor.CUSTOM_OPTIONS)

	inst:SetPoint("TOPLEFT", title, "TOPLEFT", 0, -20)
	inst:SetWidth(400)
	inst:SetText(LFMonitor.CUSTOM_INST)

	basictext:SetPoint("LEFT", inst, "TOPLEFT", 5, -30)
	basictext:SetText(LFMonitor.CUSTOM_BASIC)

	basicedit:SetPoint("LEFT", basictext, "TOPLEFT", 5, -25)
	basicedit:SetWidth(385)
	basicedit:SetHeight(32)
	basicedit:SetMaxLetters(90)
	basicedit:SetAutoFocus(false)
	basicedit:SetText(LFMonitorDB.options.basic or "")

	warn:SetPoint("TOPLEFT", basicedit, "TOPLEFT", -5, -45)
	warn:SetTextColor(1, 1, 1, 1)
	warn:SetWidth(390)
	warn:SetJustifyH("LEFT")
	warn:SetText(LFMonitor.CUSTOM_WARNING)

	advtext:SetPoint("LEFT", warn, "BOTTOMLEFT", 0, -20)
	advtext:SetText(LFMonitor.CUSTOM_ADV)

	advedit:SetPoint("LEFT", advtext, "TOPLEFT", 5, -25)
	advedit:SetWidth(385)
	advedit:SetHeight(32)
	advedit:SetMaxLetters(90)
	advedit:SetAutoFocus(false)
	advedit:SetText(LFMonitorDB.options.adv or "")

	LFMonitor.cchantext:SetPoint("TOPLEFT", advedit, "TOPLEFT", 0, -35)
	LFMonitor.cchantext:SetWidth(400)
	LFMonitor.cchantext:SetText(LFMonitor.CUSTOM_CHANNELS)

	local function ctlOnClick(idx)

		LFMonitorDB.channels["cc" .. LFMonitor["cc" .. idx .. "text"]:GetText()] = LFMonitor["cc" .. idx]:GetChecked()

	end

	for ctl = 1,7 do

		if (ctl == 1) then

			LFMonitor["cc1"]:SetPoint("TOPLEFT", LFMonitor.cchantext, "BOTTOMLEFT", 5, -5)

		else

			LFMonitor["cc" .. ctl]:SetPoint("TOPLEFT", LFMonitor["cc" .. (ctl - 1)], "BOTTOMLEFT", 0, 0)

		end

		LFMonitor["cc" .. ctl .. "text"]:SetPoint("TOPLEFT", LFMonitor["cc" .. ctl], "TOPRIGHT", 5, -5)
		LFMonitor["cc" .. ctl]:SetScript("OnClick", function() ctlOnClick(ctl) end)

	end

	return frame

end
function LFMonitor:CreateOptions()

	LFMonitor:CheckSettingsVersion(true)

	local mainPanel = CreateMainPanel()
	local responsePanel = CreateResponsePanel()
	local customPanel = CreateCustomPanel()

	mainPanel.name = "LFMonitor"
	mainPanel.default = function() StaticPopup_Show("LFMONITOR_DEFAULTS") end
	InterfaceOptions_AddCategory(mainPanel)

	AceConfig:RegisterOptionsTable("LFMonitor General", LFMonitor.GeneralOptionTable)
	AceConfigDialog:AddToBlizOptions("LFMonitor General", _G.GENERAL_LABEL, "LFMonitor")

	AceConfig:RegisterOptionsTable("LFMonitor Appearance", LFMonitor.AppearanceOptionTable)
	AceConfigDialog:AddToBlizOptions("LFMonitor Appearance", _G.APPEARANCE_LABEL, "LFMonitor")

	AceConfig:RegisterOptionsTable("LFMonitor Filtering", LFMonitor.FilterOptionTable)
	AceConfigDialog:AddToBlizOptions("LFMonitor Filtering", loc["Filtering"], "LFMonitor")

	responsePanel.name = LFMonitor.RESPONSES
	responsePanel.parent = "LFMonitor"
	responsePanel.okay = function() LFMonitor:ResponseOptionsOkay() end
	responsePanel.cancel = function() LFMonitor:BuildTables() end
	responsePanel.default = function() StaticPopup_Show("LFMONITOR_DEFAULTS") end
	InterfaceOptions_AddCategory(responsePanel)

	customPanel.name = LFMonitor.CUSTOM_OPTIONS
	customPanel.parent = "LFMonitor"
	customPanel.default = function() StaticPopup_Show("LFMONITOR_DEFAULTS") end
	customPanel.okay = function() LFMonitor:CustomOptionsOkay() end

	InterfaceOptions_AddCategory(customPanel)

end
